use chrono::{DateTime, Utc};
use sqlx::{Error, PgPool};
use sqlx::types::{chrono, JsonValue};

#[derive(Debug, Clone, Default)]
pub struct Account {
    pub id: String,
    pub username: String,
    pub display_name: Option<String>,
    pub avatar_url: Option<String>,
    pub lang_tag: String,
    pub location: Option<String>,
    pub timezone: Option<String>,
    pub metadata: JsonValue,
    pub wallet: JsonValue,
    pub email: Option<String>,
    pub password: Option<Vec<u8>>,
    pub facebook_id: Option<String>,
    pub google_id: Option<String>,
    pub gamecenter_id: Option<String>,
    pub steam_id: Option<String>,
    pub custom_id: Option<String>,
    pub apple_id: Option<String>,
    pub facebook_instant_game_id: Option<String>,
    pub weixin_id: Option<String>,
    pub douyin_id: Option<String>,
    pub create_time: DateTime<Utc>,
    pub update_time: DateTime<Utc>,
    pub verify_time: DateTime<Utc>,
    pub disable_time: DateTime<Utc>,
}

impl Account {

    pub async fn find_by_apple_id(apple_id: &str, db_pool: &PgPool) -> Result<Option<Account>, Error> {
        match sqlx::query_as!(
            Account,
            r#"select * from account where apple_id = $1"#,
            apple_id
        )
            .fetch_one(db_pool)
            .await {
            Ok(account) => {
                return Ok(Some(account));
            }
            Err(Error::RowNotFound) => {
                return Err(Error::RowNotFound);
            }
            Err(e) => {
                tracing::error!("find_by_apple_id error: {:?}", e);
                return Err(e);
            }
        }
    }

    pub async fn find_by_custom_id(custom_id: &str, db_pool: &PgPool) -> Result<Option<Account>, Error> {
        match sqlx::query_as!(
            Account,
            r#"select * from account where custom_id = $1"#,
            custom_id
        )
            .fetch_one(db_pool)
            .await {
            Ok(account) => {
                return Ok(Some(account));
            }
            Err(Error::RowNotFound) => {
                return Err(Error::RowNotFound);
            }
            Err(e) => {
                tracing::error!("find_by_custom_id error: {:?}", e);
                return Err(e);
            }
        }
    }

    pub async fn find_by_facebook_id(facebook_id: &str, db_pool: &PgPool) -> Result<Option<Account>, Error> {
        match sqlx::query_as!(
            Account,
            r#"select * from account where facebook_id = $1"#,
            facebook_id
        )
            .fetch_one(db_pool)
            .await {
            Ok(account) => {
                return Ok(Some(account));
            }
            Err(Error::RowNotFound) => {
                return Err(Error::RowNotFound);
            }
            Err(e) => {
                tracing::error!("find_by_facebook_id error: {:?}", e);
                return Err(e);
            }
        }
    }
    pub async fn find_by_google_id(google_id: &str, db_pool: &PgPool) -> Result<Option<Account>, Error> {
        match sqlx::query_as!(
            Account,
            r#"select * from account where google_id = $1"#,
            google_id
        )
            .fetch_one(db_pool)
            .await {
            Ok(account) => {
                return Ok(Some(account));
            }
            Err(Error::RowNotFound) => {
                return Ok(None);
            }
            Err(e) => {
                tracing::error!("find_by_google_id error: {:?}", e);
                return Err(e);
            }
        }
    }

    pub async fn save_google_account(&self, db_pool: &PgPool) -> Result<Account, Error> {
        sqlx::query_as!(
            Account,
            r#"
               insert into account
               (username, google_id, email, display_name, avatar_url)
               values
               ($1, $2, $3, $4, $5) returning *
            "#,
            self.username,
            self.google_id.clone().unwrap(),
            self.email.clone().unwrap(),
            self.display_name.clone().unwrap(),
            self.avatar_url.clone().unwrap()
        ).fetch_one(db_pool).await
    }
}